@@ -30,7 +30,7 @@
 #include <libgda.h>
 #include <libgda/gda-attributes-manager.h>
 #include <libgda/gda-custom-marshal.h>
-#include <libgda/gda-error.h>
+#include <libgda/gda-types.h>
 
 /* 
  * Main static functions 
@@ -176,6 +176,12 @@ gda_holder_class_init (GdaHolderClass *class)
 
 	parent_class = g_type_class_peek_parent (class);
 
+	/**
+	 * GdaHolder::source-changed
+	 * @holder: the #GdaHolder
+	 * 
+	 * Gets emitted when the data model in which @holder's values should be has changed
+	 */
 	gda_holder_signals[SOURCE_CHANGED] =
                 g_signal_new ("source-changed",
                               G_TYPE_FROM_CLASS (object_class),
@@ -183,6 +189,12 @@ gda_holder_class_init (GdaHolderClass *class)
                               G_STRUCT_OFFSET (GdaHolderClass, source_changed),
                               NULL, NULL,
                               _gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
+	/**
+	 * GdaHolder::changed
+	 * @holder: the #GdaHolder
+	 * 
+	 * Gets emitted when @holder's value has changed
+	 */
 	gda_holder_signals[CHANGED] =
                 g_signal_new ("changed",
                               G_TYPE_FROM_CLASS (object_class),
@@ -190,6 +202,14 @@ gda_holder_class_init (GdaHolderClass *class)
                               G_STRUCT_OFFSET (GdaHolderClass, changed),
                               NULL, NULL,
                               _gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
+	/**
+	 * GdaHolder::attribute-changed
+	 * @holder: the #GdaHolder
+	 * @att_name: attribute's name
+	 * @att_value: attribute's value
+	 * 
+	 * Gets emitted when any @holder's attribute has changed
+	 */
 	gda_holder_signals[ATT_CHANGED] =
                 g_signal_new ("attribute-changed",
                               G_TYPE_FROM_CLASS (object_class),
@@ -200,7 +220,7 @@ gda_holder_class_init (GdaHolderClass *class)
 			      G_TYPE_STRING, G_TYPE_VALUE);
 
 	/**
-	 * GdaHolder::before-change:
+	 * GdaHolder::validate-change
 	 * @holder: the object which received the signal
 	 * @new_value: the proposed new value for @holder
 	 * 
@@ -231,34 +251,39 @@ gda_holder_class_init (GdaHolderClass *class)
 	object_class->set_property = gda_holder_set_property;
 	object_class->get_property = gda_holder_get_property;
 	g_object_class_install_property (object_class, PROP_ID,
-					 g_param_spec_string ("id", NULL, NULL, NULL, 
+					 g_param_spec_string ("id", NULL, "Holder's ID", NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_NAME,
-					 g_param_spec_string ("name", NULL, NULL, NULL, 
+					 g_param_spec_string ("name", NULL, "Holder's name", NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_DESCR,
-					 g_param_spec_string ("description", NULL, NULL, NULL, 
+					 g_param_spec_string ("description", NULL, "Holder's description", NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_GDA_TYPE,
-					 g_param_spec_gtype ("g-type", NULL, NULL, G_TYPE_NONE, 
+					 g_param_spec_gtype ("g-type", NULL, "Holder's GType", G_TYPE_NONE, 
 							     (G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT)));
 	g_object_class_install_property (object_class, PROP_NOT_NULL,
-					 g_param_spec_boolean ("not-null", NULL, NULL, FALSE,
+					 g_param_spec_boolean ("not-null", NULL, "Can the value holder be NULL?", FALSE,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_SIMPLE_BIND,
-					 g_param_spec_object ("simple-bind", NULL, NULL, 
+					 g_param_spec_object ("simple-bind", NULL, 
+							      "Make value holder follow other GdaHolder's changes", 
                                                                GDA_TYPE_HOLDER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_FULL_BIND,
-					 g_param_spec_object ("full-bind", NULL, NULL, 
+					 g_param_spec_object ("full-bind", NULL,
+							      "Make value holder follow other GdaHolder's changes "
+							      "and the other way around", 
                                                                GDA_TYPE_HOLDER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_SOURCE_MODEL,
-                                         g_param_spec_object ("source-model", NULL, NULL,
-                                                               GDA_TYPE_DATA_MODEL,
+                                         g_param_spec_object ("source-model", NULL, "Data model among which the holder's "
+							      "value should be",
+							      GDA_TYPE_DATA_MODEL,
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property (object_class, PROP_SOURCE_COLUMN,
-                                         g_param_spec_int ("source-column", NULL, NULL,
+                                         g_param_spec_int ("source-column", NULL, "Column number to use in coordination "
+							   "with the source-model property",
 							   0, G_MAXINT, 0,
 							   (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	
@@ -375,8 +400,8 @@ gda_holder_copy (GdaHolder *orig)
  * @id: the id of the holder to create, or %NULL
  * @...: value to set
  *
- * Creates a new #GdaHolder object named @name, of type @type, and containing the value passed
- * as the last argument.
+ * Creates a new #GdaHolder object with an ID set to @id, of type @type, 
+ * and containing the value passed as the last argument.
  *
  * Note that this function is a utility function and that anly a limited set of types are supported. Trying
  * to use an unsupported type will result in a warning, and the returned value holder holding a safe default
@@ -1359,7 +1384,7 @@ gda_holder_set_default_value (GdaHolder *holder, const GValue *value)
 /**
  * gda_holder_set_not_null
  * @holder: a #GdaHolder object
- * @not_null:
+ * @not_null: TRUE if @holder should not accept %NULL values
  *
  * Sets if the holder can have a NULL value. If @not_null is TRUE, then that won't be allowed
  */
@@ -1396,8 +1421,9 @@ gda_holder_get_not_null (GdaHolder *holder)
  * @col: the reference column in @model
  * @error: location to store error, or %NULL
  *
- * Sets a limit on the possible values for the @holder holder: the value must be among the values
- * contained in the @col column of the @model data model.
+ * Sets an hint that @holder's values should be restricted among the values
+ * contained in the @col column of the @model data model. Note that this is just a hint,
+ * meaning this policy is not enforced by @holder's implementation.
  *
  * Returns: TRUE if no error occurred
  */
@@ -1444,12 +1470,16 @@ gda_holder_set_source_model (GdaHolder *holder, GdaDataModel *model,
 /**
  * gda_holder_get_source_model
  * @holder: a #GdaHolder
- * @col: a place to store the column in the model sourceing the holder, or %NULL
+ * @col: a place to store the column in the model sourcing the holder, or %NULL
  *
- * Tells if @holder has its values sourceed by a #GdaDataModel, and optionnaly
- * allows to fetch the resteictions.
+ * If gda_holder_set_source_model() has been used to provide a hint that @holder's value
+ * should be among the values contained in a column of a data model, then this method
+ * returns which data model, and if @col is not %NULL, then it is set to the restricting column
+ * as well.
  *
- * Returns: a pointer to the #GdaDataModel source for @holder
+ * Otherwise, this method returns %NULL, and if @col is not %NULL, then it is set to 0.
+ *
+ * Returns: a pointer to a #GdaDataModel, or %NULL
  */
 GdaDataModel *
 gda_holder_get_source_model (GdaHolder *holder, gint *col)
@@ -1467,10 +1497,13 @@ gda_holder_get_source_model (GdaHolder *holder, gint *col)
  * gda_holder_set_bind
  * @holder: a #GdaHolder
  * @bind_to: a #GdaHolder or %NULL
+ * @error: a place to store errors, or %NULL
  *
  * Sets @holder to change when @bind_to changes (and does not make @bind_to change when @holder changes).
  *
  * If @bind_to is %NULL, then @holder will not be bound anymore.
+ *
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_holder_set_bind (GdaHolder *holder, GdaHolder *bind_to, GError **error)
@@ -1616,17 +1649,17 @@ full_bind_changed_cb (GdaHolder *alias_of, GdaHolder *holder)
 {
 	if (alias_of == holder->priv->simple_bind) {
 		const GValue *cvalue;
-		GError *error = NULL;
+		GError *lerror = NULL;
 		cvalue = gda_holder_get_value (alias_of);
-		if (! gda_holder_set_value (holder, cvalue, &error)) {
-			g_warning (_("Could not change GdaHolder to match value change in bound GdaHolder: %s"),
-				   error && error->message ? error->message : _("No detail"));
-			if (error)
-				g_error_free (error);
+		if (! gda_holder_set_value (holder, cvalue, &lerror)) {
+			if (lerror && ((lerror->domain != GDA_HOLDER_ERROR) || (lerror->code != GDA_HOLDER_VALUE_NULL_ERROR)))
+				g_warning (_("Could not change GdaHolder to match value change in bound GdaHolder: %s"),
+					   lerror && lerror->message ? lerror->message : _("No detail"));
+			g_clear_error (&lerror);
 		}
 	}
 	else
-		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
+		g_signal_emit (holder, gda_holder_signals [CHANGED], 0);
 }
 
 /**
