@@ -18,6 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#undef GDA_DISABLE_DEPRECATED
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-statement.h>
@@ -560,6 +561,19 @@ static gchar *default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderin
 static gchar *default_render_select_from (GdaSqlSelectFrom *from, GdaSqlRenderingContext *context, GError **error);
 static gchar *default_render_select_order (GdaSqlSelectOrder *order, GdaSqlRenderingContext *context, GError **error);
 
+/**
+ * gda_statement_to_sql_real
+ * @stmt: a #GdaStatement object
+ * @context: a #GdaSqlRenderingContext context
+ * @error: a place to store errors, or %NULL
+ *
+ * Renders @stmt to its SQL representation, using @context to specify how each part of @stmt must
+ * be rendered. This function is mainly used by database provider's implementations which require
+ * to specialize some aspects of SQL rendering to be adapted to the database,'s own SQL dialect
+ * (for example SQLite rewrites the 'FALSE' and 'TRUE' literals as '0' and 'NOT 0').
+ * 
+ * Returns: a new string, or %NULL if an error occurred
+ */
 gchar *
 gda_statement_to_sql_real (GdaStatement *stmt, GdaSqlRenderingContext *context, GError **error)
 {
@@ -770,6 +784,7 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 	GString *string;
 	gchar *str;
 	GSList *list;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (stmt, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (stmt)->type == GDA_SQL_ANY_STMT_INSERT, NULL);
@@ -803,7 +818,10 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 
 	/* values */
 	if (stmt->select) {
-		g_string_append_c (string, ' ');
+		if (pretty)
+			g_string_append_c (string, '\n');
+		else
+			g_string_append_c (string, ' ');
 		str = context->render_select (GDA_SQL_ANY_PART (stmt->select), context, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -812,8 +830,12 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 	else {
 		for (list = stmt->values_list; list; list = list->next) {
 			GSList *rlist;
-			if (list == stmt->values_list)
-				g_string_append (string, " VALUES");
+			if (list == stmt->values_list) {
+				if (pretty)
+					g_string_append (string, "\nVALUES");
+				else
+					g_string_append (string, " VALUES");
+			}
 			else
 				g_string_append_c (string, ',');
 			for (rlist = (GSList*) list->data; rlist; rlist = rlist->next) {
@@ -823,6 +845,8 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 					g_string_append (string, ", ");
 				str = context->render_expr ((GdaSqlExpr*) rlist->data, context, NULL, NULL, error);
 				if (!str) goto err;
+				if (pretty && (rlist != (GSList*) list->data))
+					g_string_append (string, "\n\t");
 				g_string_append (string, str);
 				g_free (str);
 			}
@@ -881,6 +905,7 @@ default_render_update (GdaSqlStatementUpdate *stmt, GdaSqlRenderingContext *cont
 	GString *string;
 	gchar *str;
 	GSList *flist, *elist;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (stmt, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (stmt)->type == GDA_SQL_ANY_STMT_UPDATE, NULL);
@@ -897,10 +922,18 @@ default_render_update (GdaSqlStatementUpdate *stmt, GdaSqlRenderingContext *cont
 	g_free (str);
 
 	/* columns set */
-	g_string_append (string, " SET ");
-	for (flist = stmt->fields_list, elist = stmt->expr_list; flist && elist; flist = flist->next, elist = elist->next) {
-		if (flist != stmt->fields_list)
+	if (pretty)
+		g_string_append (string, "\nSET ");
+	else
+		g_string_append (string, " SET ");
+	for (flist = stmt->fields_list, elist = stmt->expr_list;
+	     flist && elist;
+	     flist = flist->next, elist = elist->next) {
+		if (flist != stmt->fields_list) {
 			g_string_append (string, ", ");
+			if (pretty)
+				g_string_append (string, "\n\t");
+		}
 		str = context->render_field (GDA_SQL_ANY_PART (flist->data), context, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -914,7 +947,10 @@ default_render_update (GdaSqlStatementUpdate *stmt, GdaSqlRenderingContext *cont
 
 	/* cond */
 	if (stmt->cond) {
-		g_string_append (string, " WHERE ");
+		if (pretty)
+			g_string_append (string, "\nWHERE ");
+		else
+			g_string_append (string, " WHERE ");
 		str = context->render_expr (stmt->cond, context, NULL, NULL, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -1003,6 +1039,7 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 	GString *string;
 	gchar *str;
 	GSList *list;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (stmt, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (stmt)->type == GDA_SQL_ANY_STMT_SELECT, NULL);
@@ -1019,6 +1056,8 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 			g_string_append_c (string, ' ');
 			g_free (str);
 		}
+		if (pretty)
+			g_string_append_c (string, '\n');
 	}
 	
 	/* selected expressions */
@@ -1027,6 +1066,8 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 		if (!str) goto err;
 		if (list != stmt->expr_list)
 			g_string_append (string, ", ");
+		if (pretty)
+			g_string_append (string, "\n\t");
 		g_string_append (string, str);
 		g_free (str);
 	}
@@ -1035,14 +1076,20 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 	if (stmt->from) {
 		str = context->render_select_from (GDA_SQL_ANY_PART (stmt->from), context, error);
 		if (!str) goto err;
-		g_string_append_c (string, ' ');
+		if (pretty)
+			g_string_append_c (string, '\n');
+		else
+			g_string_append_c (string, ' ');
 		g_string_append (string, str);
 		g_free (str);
 	}
 
 	/* WHERE */
 	if (stmt->where_cond) {
-		g_string_append (string, " WHERE ");
+		if (pretty)
+			g_string_append (string, "\nWHERE ");
+		else
+			g_string_append (string, " WHERE ");
 		str = context->render_expr (stmt->where_cond, context, NULL, NULL, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -1055,15 +1102,22 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 		if (!str) goto err;
 		if (list != stmt->group_by)
 			g_string_append (string, ", ");
-		else
-			g_string_append (string, " GROUP BY ");
+		else {
+			if (pretty)
+				g_string_append (string, "\nGROUP BY ");
+			else
+				g_string_append (string, " GROUP BY ");
+		}
 		g_string_append (string, str);
 		g_free (str);
 	}
 
 	/* HAVING */
 	if (stmt->having_cond) {
-		g_string_append (string, " HAVING ");
+		if (pretty)
+			g_string_append (string, "\nHAVING ");
+		else
+			g_string_append (string, " HAVING ");
 		str = context->render_expr (stmt->having_cond, context, NULL, NULL, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -1076,8 +1130,12 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 		if (!str) goto err;
 		if (list != stmt->order_by)
 			g_string_append (string, ", ");
-		else
-			g_string_append (string, " ORDER BY ");
+		else {
+			if (pretty)
+				g_string_append (string, "\nORDER BY ");
+			else
+				g_string_append (string, " ORDER BY ");
+		}
 		g_string_append (string, str);
 		g_free (str);
 	}
@@ -1279,7 +1337,8 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 }
 
 static gchar *
-default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean *is_default, gboolean *is_null, GError **error)
+default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean *is_default,
+		     gboolean *is_null, GError **error)
 {
 	GString *string;
 	gchar *str = NULL;
@@ -1303,13 +1362,45 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		if (!str) goto err;
 	}
 	else if (expr->value) {
-		str = gda_value_stringify (expr->value);
-		if (!str) goto err;
-		if (is_null && gda_value_is_null (expr->value))
-			*is_null = TRUE;
-		else if (is_default && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) && 
-			 !g_ascii_strcasecmp (g_value_get_string (expr->value), "default"))
-			*is_default = TRUE;
+		if (expr->value_is_ident && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) &&
+		    g_value_get_string (expr->value)) {
+			gchar **ids_array;
+			gint i;
+			GString *string = NULL;
+			GdaConnectionOptions cncoptions = 0;
+			if (context->cnc)
+				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+
+			ids_array = gda_sql_identifier_split (g_value_get_string (expr->value));
+			if (!ids_array) 
+				str = g_value_dup_string (expr->value);
+			else if (!(ids_array[0])) goto err;
+			else {
+				for (i = 0; ids_array[i]; i++) {
+					gchar *tmp;
+					if (!string)
+						string = g_string_new ("");
+					else
+						g_string_append_c (string, '.');
+					tmp = gda_sql_identifier_quote (ids_array[i], context->cnc,
+									context->provider, FALSE,
+					   cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+					g_string_append (string, tmp);
+					g_free (tmp);
+				}
+				g_strfreev (ids_array);
+				str = g_string_free (string, FALSE);
+			}
+		}
+		else {
+			str = gda_value_stringify (expr->value);
+			if (!str) goto err;
+			if (is_null && gda_value_is_null (expr->value))
+				*is_null = TRUE;
+			else if (is_default && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) && 
+				 !g_ascii_strcasecmp (g_value_get_string (expr->value), "default"))
+				*is_default = TRUE;
+		}
 	}
 	else if (expr->func) {
 		str = context->render_function (GDA_SQL_ANY_PART (expr->func), context, error);
@@ -1323,8 +1414,14 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		gchar *str1;
 		str1 = context->render_select (GDA_SQL_ANY_PART (expr->select), context, error);
 		if (!str1) goto err;
-		str = g_strdup_printf ("(%s)", str1);
-		g_free (str1);
+
+		if (! GDA_SQL_ANY_PART (expr)->parent ||
+		    (GDA_SQL_ANY_PART (expr)->parent->type != GDA_SQL_ANY_SQL_FUNCTION)) {
+			str = g_strdup_printf ("(%s)", str1);
+			g_free (str1);
+		}
+		else
+			str = str1;
 	}
 	else if (expr->case_s) {
 		str = context->render_case (GDA_SQL_ANY_PART (expr->case_s), context, error);
@@ -1365,7 +1462,12 @@ default_render_field (GdaSqlField *field, GdaSqlRenderingContext *context, GErro
 	/* can't have: field->field_name not a valid SQL identifier */
 	if (!gda_sql_any_part_check_structure (GDA_SQL_ANY_PART (field), error)) return NULL;
 
-	return g_strdup (field->field_name);
+	GdaConnectionOptions cncoptions = 0;
+	if (context->cnc)
+		g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+	return gda_sql_identifier_quote (field->field_name, context->cnc, context->provider,
+					 FALSE,
+					 cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
 }
 
 static gchar *
@@ -1377,7 +1479,33 @@ default_render_table (GdaSqlTable *table, GdaSqlRenderingContext *context, GErro
 	/* can't have: table->table_name not a valid SQL identifier */
 	if (!gda_sql_any_part_check_structure (GDA_SQL_ANY_PART (table), error)) return NULL;
 
-	return g_strdup (table->table_name);
+	gchar **ids_array;
+	ids_array = gda_sql_identifier_split (table->table_name);
+	if (!ids_array) {
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+			     "%s", _("Malformed table name"));
+		return NULL;
+	}
+	
+	gint i;
+	GString *string;
+	GdaConnectionOptions cncoptions = 0;
+	if (context->cnc)
+		g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+	string = g_string_new ("");
+	for (i = 0; ids_array [i]; i++) {
+		gchar *tmp;
+		tmp = gda_sql_identifier_quote (ids_array [i], context->cnc, context->provider,
+						FALSE,
+						cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+		g_free (ids_array [i]);
+		ids_array [i] = tmp;
+		if (i != 0)
+			g_string_append_c (string, '.');
+		g_string_append (string, ids_array [i]);
+	}
+	g_strfreev (ids_array);
+	return g_string_free (string, FALSE);
 }
 
 static gchar *
@@ -1394,7 +1522,7 @@ default_render_function (GdaSqlFunction *func, GdaSqlRenderingContext *context,
 	if (!gda_sql_any_part_check_structure (GDA_SQL_ANY_PART (func), error)) return NULL;
 
 	string = g_string_new (func->function_name);
-	g_string_append_c (string, '(');
+	g_string_append (string, " (");
 	for (list = func->args_list; list; list = list->next) {
 		if (list != func->args_list)
 			g_string_append (string, ", ");
@@ -1711,6 +1839,7 @@ default_render_select_target (GdaSqlSelectTarget *target, GdaSqlRenderingContext
 {
 	GString *string;
 	gchar *str;
+	gpointer tmp;
 
 	g_return_val_if_fail (target, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (target)->type == GDA_SQL_ANY_SQL_SELECT_TARGET, NULL);
@@ -1718,11 +1847,21 @@ default_render_select_target (GdaSqlSelectTarget *target, GdaSqlRenderingContext
 	/* can't have: target->expr == NULL */
 	if (!gda_sql_any_part_check_structure (GDA_SQL_ANY_PART (target), error)) return NULL;
 
-	string = g_string_new ("");
-	str = context->render_expr (target->expr, context, NULL, NULL, error);
-	if (!str) goto err;
-	g_string_append (string, str);
-	g_free (str);
+	if (! target->expr->value || (G_VALUE_TYPE (target->expr->value) != G_TYPE_STRING)) {
+		str = context->render_expr (target->expr, context, NULL, NULL, error);
+		if (!str)
+			return NULL;
+		string = g_string_new (str);
+		g_free (str);
+	}
+	else {
+		tmp = target->expr->value_is_ident;
+		target->expr->value_is_ident = (gpointer) 0x1;
+		str = context->render_expr (target->expr, context, NULL, NULL, error);
+		target->expr->value_is_ident = tmp;
+		string = g_string_new (str);
+		g_free (str);
+	}
 
 	if (target->as)
 		g_string_append_printf (string, " AS %s", target->as);
@@ -1730,10 +1869,6 @@ default_render_select_target (GdaSqlSelectTarget *target, GdaSqlRenderingContext
 	str = string->str;
 	g_string_free (string, FALSE);
 	return str;
-
- err:
-	g_string_free (string, TRUE);
-	return NULL;
 }
 
 static gchar *
@@ -1741,6 +1876,7 @@ default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *cont
 {
 	GString *string;
 	gchar *str;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (join, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (join)->type == GDA_SQL_ANY_SQL_SELECT_JOIN, NULL);
@@ -1753,22 +1889,40 @@ default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *cont
 
 	switch (join->type) {
 	case GDA_SQL_SELECT_JOIN_CROSS:
-		string = g_string_new (",");
+		if (pretty)
+			string = g_string_new (",\n\t");
+		else
+			string = g_string_new (",");
 		break;
         case GDA_SQL_SELECT_JOIN_NATURAL:
-		string = g_string_new ("NATURAL JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tNATURAL JOIN");
+		else
+			string = g_string_new ("NATURAL JOIN");
 		break;
         case GDA_SQL_SELECT_JOIN_INNER:
-		string = g_string_new ("INNER JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tINNER JOIN");
+		else
+			string = g_string_new ("INNER JOIN");
 		break;
         case GDA_SQL_SELECT_JOIN_LEFT:
-		string = g_string_new ("LEFT JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tLEFT JOIN");
+		else
+			string = g_string_new ("LEFT JOIN");
 		break;
         case GDA_SQL_SELECT_JOIN_RIGHT:
-		string = g_string_new ("RIGHT JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tRIGHT JOIN");
+		else
+			string = g_string_new ("RIGHT JOIN");
 		break;
         case GDA_SQL_SELECT_JOIN_FULL:
-		string = g_string_new ("FULL JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tFULL JOIN");
+		else
+			string = g_string_new ("FULL JOIN");
 		break;
 	default:
 		g_assert_not_reached ();
@@ -1790,6 +1944,7 @@ default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *cont
 	}
 	str = context->render_select_target (GDA_SQL_ANY_PART (target), context, error);
 	if (!str) goto err;
+
 	g_string_append_c (string, ' ');
 	g_string_append (string, str);
 	g_free (str);
@@ -1843,6 +1998,7 @@ default_render_select_from (GdaSqlSelectFrom *from, GdaSqlRenderingContext *cont
 	gchar *str;
 	GSList *tlist;
 	gint i;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (from, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (from)->type == GDA_SQL_ANY_SQL_SELECT_FROM, NULL);
@@ -1859,14 +2015,20 @@ default_render_select_from (GdaSqlSelectFrom *from, GdaSqlRenderingContext *cont
 		if (join) {
 			str = context->render_select_join (GDA_SQL_ANY_PART (join), context, error);
 			if (!str) goto err;
-			g_string_append_c (string, ' ');
+			if (!pretty)
+				g_string_append_c (string, ' ');
 			g_string_append (string, str);
 			g_free (str);
-			g_string_append_c (string, ' ');
+			if (!pretty)
+				g_string_append_c (string, ' ');
 		}
 		else {
-			if (tlist != from->targets)
-				g_string_append (string, ", ");
+			if (tlist != from->targets) {
+				if (pretty)
+					g_string_append (string, ",\n\t");
+				else
+					g_string_append (string, ", ");
+			}
 			str = context->render_select_target (GDA_SQL_ANY_PART (tlist->data), context, error);
 			if (!str) goto err;
 			g_string_append (string, str);
