@@ -85,6 +85,10 @@ gda_g_type_to_string (GType type)
 		return "timestamp";
 	else if (type == G_TYPE_BOOLEAN)
 		return "boolean";
+	else if (type == GDA_TYPE_BLOB)
+		return "blob";
+	else if (type == GDA_TYPE_BINARY)
+		return "binary";
 	else 
 		return g_type_name (type);
 }
@@ -104,10 +108,12 @@ gda_g_type_to_string (GType type)
  *   <listitem><para>"time" for GDA_TYPE_TIME</para></listitem>
  *   <listitem><para>"timestamp" for GDA_TYPE_TIMESTAMP</para></listitem>
  *   <listitem><para>"boolean" for G_TYPE_BOOLEAN</para></listitem>
+ *   <listitem><para>"blob" for GDA_TYPE_BLOB</para></listitem>
+ *   <listitem><para>"binary" for GDA_TYPE_BINARY</para></listitem>
  *   <listitem><para>"null" for GDA_TYPE_NULL</para></listitem>
  * </itemizedlist>
  *
- * Returns: the #GType represented by the given @str.
+ * Returns: the #GType represented by the given @str, or #G_TYPE_INVALID if not found
  */
 GType
 gda_g_type_from_string (const gchar *str)
@@ -129,6 +135,10 @@ gda_g_type_from_string (const gchar *str)
 			type = GDA_TYPE_TIMESTAMP;
 		else if (!strcmp (str, "boolean"))
                         type = G_TYPE_BOOLEAN;
+		else if (!strcmp (str, "blob"))
+			type = GDA_TYPE_BLOB;
+		else if (!strcmp (str, "binary"))
+			type = GDA_TYPE_BINARY;
 		else if (!strcmp (str, "null"))
 			type = GDA_TYPE_NULL;
 		else
@@ -994,9 +1004,11 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 									     require_pk, error);
 		if (!ust->cond) {
 			retval = FALSE;
-			goto cleanup;
+			*update_stmt = NULL;
+			update_stmt = NULL; /* don't try anymore to build UPDATE statement */
 		}
-		GDA_SQL_ANY_PART (ust->cond)->parent = GDA_SQL_ANY_PART (ust);
+		else
+			GDA_SQL_ANY_PART (ust->cond)->parent = GDA_SQL_ANY_PART (ust);
 	}
         
 	if (delete_stmt) {
@@ -1006,14 +1018,19 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 
 		dst->table = gda_sql_table_new (GDA_SQL_ANY_PART (dst));
 		dst->table->table_name = g_strdup (tmp);
-		dst->cond = gda_compute_unique_table_row_condition_with_cnc (cnc, stsel, 
+		if (update_stmt && ust->cond)
+			dst->cond = gda_sql_expr_copy (ust->cond);
+		else
+			dst->cond = gda_compute_unique_table_row_condition_with_cnc (cnc, stsel, 
 									     GDA_META_TABLE (target->validity_meta_object),
 									     require_pk, error);
 		if (!dst->cond) {
 			retval = FALSE;
-			goto cleanup;
+			*delete_stmt = NULL;
+			delete_stmt = NULL; /* don't try anymore to build DELETE statement */
 		}
-		GDA_SQL_ANY_PART (dst->cond)->parent = GDA_SQL_ANY_PART (dst);
+		else
+			GDA_SQL_ANY_PART (dst->cond)->parent = GDA_SQL_ANY_PART (dst);
 	}
 	g_free (tmp);
 
@@ -1086,10 +1103,11 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 				     "%s", _("Could not compute any field to insert into"));
 			retval = FALSE;
-			goto cleanup;
 		}
-		ist->values_list = g_slist_append (NULL, insert_values_list);
-		ret_insert = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ist, NULL);
+		else {
+			ist->values_list = g_slist_append (NULL, insert_values_list);
+			ret_insert = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ist, NULL);
+		}
 	}
 	if (update_stmt)
 		ret_update = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ust, NULL);
@@ -1300,6 +1318,12 @@ prepare_sql_identifier_for_compare (gchar *str)
 	}
 }
 
+static gint
+cmp_func (gconstpointer a, gconstpointer b)
+{
+	return g_strcmp0 (*((gchar**) a), *((gchar**) b));
+}
+
 /**
  * gda_completion_list_get
  * @cnc: a #GdaConnection object
@@ -1323,6 +1347,8 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 	if (!cnc) 
 		return NULL;
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	if (!sql || !(*sql))
+		return NULL;
 	if (end < start)
 		return NULL;
 
@@ -1475,7 +1501,7 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 						
 						g_object_unref (m2);
 					}
-					free (str);
+					g_free (str);
 				}
 			}
 		}
@@ -1509,6 +1535,23 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 	g_free (text);
 	if (compl) {
 		if (compl->len >= 1) {
+			/* sort */
+			gint i;
+			g_array_sort (compl, cmp_func);
+
+			/* remove duplicates if any */
+			for (i = 1; i < compl->len; ) {
+				gchar *current, *before;
+				current = g_array_index (compl, gchar*, i);
+				before = g_array_index (compl, gchar*, i - 1);
+				if (!strcmp (current, before)) {
+					g_free (current);
+					g_array_remove_index (compl, i);
+				}
+				else
+					i++;
+			}
+
 			gchar **ptr;
 			ptr = (gchar**) compl->data;
 			g_array_free (compl, FALSE);
