@@ -442,9 +442,11 @@ gda_blob_free (gpointer boxed)
 	GdaBlob *blob = (GdaBlob*) boxed;
 	
 	g_return_if_fail (blob);
-		
-	if (blob->op)
+
+	if (blob->op) {
 		g_object_unref (blob->op);
+		blob->op = NULL;
+	}
 	gda_binary_free ((GdaBinary *) blob);
 }
 
@@ -464,8 +466,7 @@ gda_blob_set_op (GdaBlob *blob, GdaBlobOp *op)
 	}
 	if (op) {
 		g_return_if_fail (GDA_IS_BLOB_OP (op));
-		blob->op = op;
-		g_object_ref (op);
+		blob->op = g_object_ref (op);
 	}
 }
 
@@ -2485,79 +2486,79 @@ gda_ushort_get_type (void) {
  * @bin: a correctly filled @GdaBinary structure
  * @maxlen: a maximum len used to truncate, or 0 for no maximum length
  *
- * Converts all the non printable characters of bin->data into the \xxx representation
- * where xxx is the octal representation of the byte, and the '\' (backslash) character
+ * Converts all the non printable characters of bin->data into the "\xyz" representation
+ * where "xyz" is the octal representation of the byte, and the '\' (backslash) character
  * is converted to "\\".
  *
+ * Note that the backslash and newline characters are considered as printable characters and
+ * will not be represented by the "\xyz" representation.
+ *
+ * Use this function to get a representation as much readable by humans as possible of a binary
+ * chunk. Note that this function is internally called when transforming a binary value to
+ * a string for example when using g_value_transform() or gda_value_stringify().
+ *
  * Returns: a new string from @bin
  */
 gchar *
 gda_binary_to_string (const GdaBinary *bin, guint maxlen)
 {
 	gint nb_rewrites = 0;
-	gchar *ptr, *hold;
+	gchar *sptr, *rptr;
 	gulong realsize = MYMIN (bin->binary_length, maxlen);
 
 	gchar *retval;
 	glong offset = 0;
-	gunichar unichar;
 
 	if (!bin->data || (bin->binary_length == 0))
 		return g_strdup ("");
 
 	/* compute number of char rewrites */
-	ptr = (gchar*)bin->data;
-	while (offset < realsize) {
-		unichar = g_utf8_get_char_validated (ptr, -1);
-		if ((*ptr == '\n') ||
-		    ((*ptr != '\\') && (unichar > 0) && g_unichar_isprint (unichar))) {
-			hold = ptr;
-			ptr = g_utf8_next_char (ptr);
-			offset += ptr - hold;
-		}
-		else {
-			ptr++;
+	for (offset = 0, sptr = (gchar*) bin->data; 
+	     offset < realsize; 
+	     offset ++, sptr++) {
+		if ((*sptr != '\n') && ((*sptr == '\\') || !g_ascii_isprint (*sptr)))
 			nb_rewrites++;
-			offset ++;
-		}
 	}
 	
 	/* mem allocation and copy */
 	retval = g_malloc0 (realsize + nb_rewrites * 4 + 1);
-	memcpy (retval, bin->data, realsize);
-	ptr = retval;
+	rptr = retval;
+	sptr = (gchar*) bin->data;
 	offset = 0;
 	while (offset < realsize) {
-		unichar = g_utf8_get_char_validated (ptr, -1);
 		/* g_print (">%x<\n", (guchar) *ptr); */
-		if ((*ptr == '\n') ||
-		    ((*ptr != '\\') && (unichar > 0) && g_unichar_isprint (unichar))) {
-			hold = ptr;
-			ptr = g_utf8_next_char (ptr);
-			offset += ptr - hold;
+		if ((*sptr == '\n') || ((*sptr != '\\') && g_ascii_isprint (*sptr))) {
+			*rptr = *sptr;
+			rptr ++;
 		}
 		else {
-			if (*ptr == '\\') {
-				g_memmove (ptr+2, ptr+1, realsize - offset);
-
-				*(ptr+1) = '\\';
-				ptr += 2;
-				offset ++;
+			if (*sptr == '\\') {
+				*rptr = '\\';
+				rptr++;
+				*rptr = '\\';
+				rptr++;
 			}
 			else {
-				guchar val = *ptr;
-				g_memmove (ptr+4, ptr+1, realsize - offset);
-				*ptr = '\\';
-				*(ptr+1) = val / 49 + '0';
-				val = val % 49;
-				*(ptr+2) = val / 7 + '0';
-				val = val % 7;
-				*(ptr+3) = val + '0';
-				
-				ptr += 4;
-				offset ++;
+				guchar val = *sptr;
+
+				*rptr = '\\';
+				rptr++;
+
+				*rptr = val / 64 + '0';
+				rptr++;
+				val = val % 64;
+
+				*rptr = val / 8 + '0';
+				val = val % 8;
+				rptr++;
+
+				*rptr = val + '0';
+				rptr++;
 			}
 		}
+
+		sptr++;
+		offset ++;
 	}
 
 	return retval;
@@ -2567,7 +2568,9 @@ gda_binary_to_string (const GdaBinary *bin, guint maxlen)
  * gda_string_to_binary
  * @str: a string to convert
  *
- * Performs the reverse of gda_binary_to_string().
+ * Performs the reverse of gda_binary_to_string() (note that for any "\xyz" succession
+ * of 4 characters where "xyz" represents a valid octal value, the resulting read value will
+ * be modulo 256)
  *
  * Returns: a new #GdaBinary if no error were found in @str, or NULL otherwise
  */
@@ -2576,9 +2579,9 @@ gda_string_to_binary (const gchar *str)
 {
 	GdaBinary *bin;
 	glong len = 0, total;
-	gchar *ptr;
+	gchar *rptr;
+	const gchar *sptr;
 	gchar *retval;
-	glong offset = 0;
 
 	if (!str) {
 		bin = g_new0 (GdaBinary, 1);
@@ -2588,23 +2591,24 @@ gda_string_to_binary (const gchar *str)
 	}
 
 	total = strlen (str);
-	retval = g_memdup (str, total);
-	ptr = (gchar *) retval;
-	while (offset < total) {
-		if (*ptr == '\\') {
-			if (*(ptr+1) == '\\') {
-				g_memmove (ptr+1, ptr+2, total - offset);
-				offset += 2;
+	retval = g_new0 (gchar, total + 1);
+	sptr = str;
+	rptr = (gchar *) retval;
+
+	while (*sptr) {
+		if (*sptr == '\\') {
+			if (*(sptr+1) == '\\') {
+				*rptr = '\\';
+				sptr += 2;
 			}
 			else {
-				if ((*(ptr+1) >= '0') && (*(ptr+1) <= '9') &&
-				    (*(ptr+2) >= '0') && (*(ptr+2) <= '9') &&
-				    (*(ptr+3) >= '0') && (*(ptr+3) <= '9')) {
-					*ptr = (*(ptr+1) - '0') * 49 +
-						(*(ptr+2) - '0') * 7 +
-						*(ptr+3) - '0';
-					g_memmove (ptr+1, ptr+4, total - offset);
-					offset += 4;
+				if ((*(sptr+1) >= '0') && (*(sptr+1) <= '7') &&
+				    (*(sptr+2) >= '0') && (*(sptr+2) <= '7') &&
+				    (*(sptr+3) >= '0') && (*(sptr+3) <= '7')) {
+					*rptr = (*(sptr+1) - '0') * 64 +
+						(*(sptr+2) - '0') * 8 +
+						(*(sptr+3) - '0');
+					sptr += 4;
 				}
 				else {
 					g_free (retval);
@@ -2612,10 +2616,12 @@ gda_string_to_binary (const gchar *str)
 				}
 			}
 		}
-		else
-			offset ++;
+		else {
+			*rptr = *sptr;
+			sptr++;
+		}
 
-		ptr++;
+		rptr++;
 		len ++;
 	}
 
