@@ -26,6 +26,8 @@
 #include <libgda/gda-meta-store.h>
 #include <libgda/gda-meta-struct.h>
 
+G_BEGIN_DECLS
+
 /* error reporting */
 extern GQuark gda_sql_error_quark (void);
 #define GDA_SQL_ERROR gda_sql_error_quark ()
@@ -142,6 +144,12 @@ typedef struct {
 	/* augmenting information precision using a dictionary */
 	GdaSqlForeachFunc     check_structure_func;
 	GdaSqlForeachFunc     check_validity_func;
+
+	/* Padding for future expansion */
+	gpointer         _gda_reserved1;
+	gpointer         _gda_reserved2;
+	gpointer         _gda_reserved3;
+	gpointer         _gda_reserved4;
 } GdaSqlStatementContentsInfo;
 
 /*
@@ -152,6 +160,14 @@ typedef struct {
 	GdaConnection *cnc;
 	GdaMetaStore  *store;
 	GdaMetaStruct *mstruct;
+
+	/* Padding for future expansion */
+	gpointer         _gda_reserved1;
+	gpointer         _gda_reserved2;
+	gpointer         _gda_reserved3;
+	gpointer         _gda_reserved4;
 } GdaSqlStatementCheckValidityData;
 
+G_END_DECLS
+
 #endif
