@@ -7,8 +7,8 @@ gint
 main (int argc, char **argv) {
 	GdaDataModel *model;
 
-	gda_init ("list-sources", PACKAGE_VERSION, argc, argv);
-	setenv ("GDA_DATA_MODEL_DUMP_TITLE", "TRUE", TRUE);
+	gda_init ();
+	g_setenv ("GDA_DATA_MODEL_DUMP_TITLE", "TRUE", TRUE);
 
 	model = gda_config_list_providers ();
 	gda_data_model_dump (model, stdout);
